@@ -1,6 +1,6 @@
-/*  $Id$
+/*  xfce4-embed-plugin - Embed arbitrary windows into the Xfce panel
  *
- *  Copyright (c) 2011 David Schneider <dnschneid@gmail.com>
+ *  Copyright (C) 2012 David Schneider <dnschneid@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -831,6 +831,45 @@ embed_size_allocate (GtkSocket *socket, GdkRectangle *allocation,
 
 
 
+/* Callback used when the socket (or fake socket) needs to be painted.
+ * HACK: Let the parent plugin wrapper do the heavy lifting by convincing it to
+ * draw on our window for us.
+ */
+static gint
+embed_expose (GtkWidget *widget, GdkEvent *event, EmbedPlugin *embed)
+{
+  GtkWidget *plugwidget = gtk_widget_get_parent (GTK_WIDGET (embed->plugin));
+  GdkWindow *plugwindow = plugwidget->window;
+  plugwidget->window = widget->window;
+  gtk_widget_send_expose (plugwidget, event);
+  plugwidget->window = plugwindow;
+  return TRUE;
+}
+
+
+
+/* Callback for when the gtksocket is realized.
+ * Set the window settings.
+ */
+static void
+embed_socket_realize (GtkWidget *socket, EmbedPlugin *embed)
+{
+  /* Ensure the socket gets expose and mouse button events.
+   * It needs EXPOSURE_MASK so that it properly gets repainted.
+   * It needs press/release so that the plugin menu can appear.
+   * It needs GDK_SUBSTRUCTURE_MASK, because otherwise GtkSocket breaks.
+   */
+  GdkWindow *socketwindow = gtk_widget_get_window (socket);
+  g_assert (socketwindow);
+  gdk_window_set_events (socketwindow, gdk_window_get_events (socketwindow)
+                                       | GDK_EXPOSURE_MASK
+                                       | GDK_BUTTON_PRESS_MASK
+                                       | GDK_BUTTON_RELEASE_MASK
+                                       | GDK_SUBSTRUCTURE_MASK);
+}
+
+
+
 /* Adds a GtkSocket to the plugin and hooks up the signals, optionally updating
  * the size of the plugin to match. Generally update_size should be true unless
  * the plugin is being initialized.
@@ -851,7 +890,14 @@ embed_add_socket (EmbedPlugin *embed, gboolean update_size)
   g_signal_connect (G_OBJECT (embed->socket), "plug-removed",
                     G_CALLBACK (embed_plug_removed), embed);
   g_signal_connect (G_OBJECT (embed->socket), "realize",
-                    G_CALLBACK (embed_start_search), embed);
+                    G_CALLBACK (embed_socket_realize), embed);
+  g_signal_connect (G_OBJECT (embed->socket), "expose-event",
+                    G_CALLBACK (embed_expose), embed);
+  g_signal_connect_after (G_OBJECT (embed->socket), "realize",
+                          G_CALLBACK (embed_start_search), embed);
+
+  xfce_panel_plugin_add_action_widget (embed->plugin, embed->socket);
+  gtk_widget_set_app_paintable (embed->socket, TRUE);
 
   /* Add it to the plugin hvbox */
   gtk_widget_show (embed->socket);
@@ -879,6 +925,13 @@ embed_add_fake_socket (EmbedPlugin *embed)
   /* We use the size-allocate signal to keep the size of the plug up-to-date. */
   g_signal_connect (G_OBJECT (embed->socket), "size-allocate",
                     G_CALLBACK (embed_size_allocate), embed);
+  g_signal_connect (G_OBJECT (embed->socket), "realize",
+                    G_CALLBACK (embed_socket_realize), embed);
+  g_signal_connect (G_OBJECT (embed->socket), "expose-event",
+                    G_CALLBACK (embed_expose), embed);
+
+  xfce_panel_plugin_add_action_widget (embed->plugin, embed->socket);
+  gtk_widget_set_app_paintable (embed->socket, TRUE);
 
   /* Add it to the plugin hvbox */
   gtk_widget_show (embed->socket);
